
'S2 include code version 2010.11.18

'---[Constants]----------------------------------------------------------------

CON

  _clkmode      = xtal1 + pll16x
  _xinfreq      = 5_000_000
  EE_REFLIGHTS  = s2#EE_USER_AREA
  LINE_THLD     = 32
  BAR_THLD      = 32
  OBSTACLE_THLD = 20
  SPKR_VOL      = 35

'---[Global Variables]---------------------------------------------------------

VAR

  long  CoinFlip, LeftMotor, RightMotor, MoveTime, pLeftMotor, pRightMotor, pMoveTime, FMStack[50]
  word  WheelSpace, FullCircle, SeqCounter
  byte  LeftLight, CenterLight, RightLight, RefLights[3]
  byte  ResetCount, LineCount, ObstacleCount, StallCount, ObstacleThld
  byte  LineThld, LeftLine, RightLine, LeftObstacle, RightObstacle, Self
  byte  Flag_green, Flag_yellow, Flag_orange, Flag_red, Flag_magenta, Flag_purple, Flag_blue, Stalled

  long ledstack[10]             ' Stack space for blink method

'---[Object Declaration]-------------------------------------------------------

OBJ

  s2    : "S2"

'---[Start of Program]---------------------------------------------------------

PUB start

  s2.start
  s2.start_motors
  s2.start_tones
  s2.button_mode(true, true)
  s2.set_volume(SPKR_VOL)
  s2.set_voices(s2#SAW, s2#SAW)
  ResetCount := s2.reset_button_count
  if (s2.get_line_threshold <> s2#DEFAULT_LINE_THLD)
    LineThld := s2.get_line_threshold
  else
    LineThld := LINE_THLD 
  if (s2.get_obstacle_threshold <> s2#DEFAULT_OBSTACLE_THLD)
    ObstacleThld := s2.get_obstacle_threshold
  else
    ObstacleThld := OBSTACLE_THLD 
  waitcnt(cnt + 10_000_000) 
  CoinFlip :=  s2.light_sensor_raw(s2#LEFT) << 24 | s2.light_sensor_raw(s2#CENTER) << 12 | s2.light_sensor_raw(s2#RIGHT)
  Self := cogid
  long[@WheelSpace] := s2.get_wheel_calibration
  cognew(FaultMonitor, @FMStack)
  \Green
  repeat

'---[Battery and Over-current Monitor Cog]-------------------------------------

PUB FaultMonitor : value

  value := $ffff
  waitcnt(cnt + 80_000_000)
  repeat
    value <#= s2.get_adc_results(s2#ADC_VBAT)
    if value > constant((700*2550)/(400*33))      '7.0V
      s2.set_led(s2#POWER,s2#BLUE)
    elseif value > constant((600*2550)/(400*33))  '6.0V
      s2.set_led(s2#POWER,$20)
    else
      s2.set_led(s2#POWER,s2#BLINK_BLUE)
    if s2.get_adc_results(s2#ADC_IMOT) > 210
      cogstop(Self)
      s2.stop_now
      s2.set_leds(s2#BLINK_RED,s2#BLINK_RED,s2#BLINK_RED,s2#OFF)
      repeat

'---[Main Program: Green]------------------------------------------------------

PUB Green

  repeat
    ReadObstacle
    ReadStall
    if (LeftObstacle == 1 and RightObstacle == 1)
      MotorSet(0, 0, 0)
      cognew(Blink, @ledstack)  ' Call light blinking code
      Yellow
      MotorSet(-128, -128, 1000)
      MotorSet(-128, 128, 1000)
    elseif (LeftObstacle == 1 and RightObstacle == 0)
      MotorSet(0, 128, 0)
    elseif (LeftObstacle == 0 and RightObstacle == 1)
      MotorSet(128, 0, 0)
    elseif (Stalled == 1)
      MotorSet(-128, -128, 0)
      MotorSet(128, -128, 0)
    else
      MotorSet(128, 128, 0)

'---[Subroutine: Yellow]-------------------------------------------------------

PUB Yellow

  PlayNote(457, 261, 0)
  PlayNote(343, 349, 0)
  PlayNote(228, 349, 0)
  PlayNote(457, 349, 0)
  PlayNote(457, 391, 0)
  PlayNote(343, 440, 0)
  PlayNote(228, 440, 0)
  PlayNote(686, 440, 0)
  PlayNote(228, 349, 0)
  PlayNote(228, 391, 0)
  PlayNote(228, 440, 0)
  PlayNote(457, 466, 0)
  PlayNote(457, 329, 0)
  PlayNote(457, 391, 0)
  PlayNote(457, 349, 0)
  PlayNote(228, 0, 0)
  s2.wait_sync(0)
  PlayNote(457, 261, 0)
  PlayNote(343, 349, 0)
  PlayNote(228, 349, 0)
  PlayNote(457, 349, 0)
  PlayNote(457, 391, 0)
  PlayNote(343, 440, 0)
  PlayNote(228, 440, 0)
  PlayNote(686, 440, 0)
  PlayNote(228, 349, 0)
  PlayNote(228, 391, 0)
  PlayNote(228, 440, 0)
  PlayNote(457, 466, 0)
  PlayNote(457, 329, 0)
  PlayNote(457, 391, 0)
  PlayNote(457, 349, 0)
  PlayNote(228, 0, 0)
  s2.wait_sync(0)
  PlayNote(228, 523, 0)
  PlayNote(228, 523, 0)
  PlayNote(228, 440, 0)
  PlayNote(686, 587, 0)
  PlayNote(228, 523, 0)
  PlayNote(228, 523, 0)
  PlayNote(228, 466, 0)
  PlayNote(686, 466, 0)
  PlayNote(228, 466, 0)
  PlayNote(228, 466, 0)
  PlayNote(228, 391, 0)
  PlayNote(686, 523, 0)
  PlayNote(228, 466, 0)
  PlayNote(228, 466, 0)
  PlayNote(228, 440, 0)
  PlayNote(457, 440, 0)
  s2.wait_sync(0)
  PlayNote(457, 261, 0)
  PlayNote(343, 349, 0)
  PlayNote(228, 349, 0)
  PlayNote(457, 349, 0)
  PlayNote(457, 391, 0)
  PlayNote(343, 440, 0)
  PlayNote(228, 440, 0)
  PlayNote(686, 440, 0)
  PlayNote(228, 349, 0)
  PlayNote(228, 391, 0)
  PlayNote(228, 440, 0)
  PlayNote(457, 466, 0)
  PlayNote(457, 329, 0)
  PlayNote(457, 391, 0)
  PlayNote(457, 349, 0)
  PlayNote(228, 0, 0)
  s2.wait_sync(0)

PUB Blink              

  dira[5..2]~~                  ' Set pins 2-5 to outputs

  repeat 23                     ' Repeat subsequent code 23 times                                              
    outa[5..2] := %1010         ' Turn LED pins 5 & 3 ON and LED pins 4 & 2 OFF
    waitcnt(clkfreq / 2 + cnt)  ' Wait 0.5 s
    outa[5..2] := %0101         ' Turn LED pins 4 & 2 OFF and LED pins 5 & 3 ON
    waitcnt(clkfreq / 2 + cnt)  ' Wait 0.5 s

'---[Set Motor Speeds]---------------------------------------------------------

PRI MotorSet(lmotor, rmotor, timer)

  MoveTime := timer #> 0
  LeftMotor := lmotor #> -256 <# 256
  RightMotor := rmotor #> -256 <# 256
  if (LeftMotor <> pLeftMotor or RightMotor <> pRightMotor or MoveTime <> pMoveTime or pMoveTime <> 0)
    if (MoveTime)
      s2.move_now(LeftMotor * MoveTime * FullCircle / 1_024_000, RightMotor * MoveTime * FullCircle / 1_024_000, MoveTime << 1, (||LeftMotor #> ||RightMotor <# 255) >> 4, 0)
      s2.wait_stop
    else  
      s2.wheels_now(LeftMotor, RightMotor, 0)
      waitcnt(cnt + clkfreq / 10)
    pLeftMotor := LeftMotor
    pRightMotor := RightMotor
    pMoveTime := MoveTime

'---[Play an Individual Note, Separated by a Short Delay.]---------------------

PRI PlayNote(duration, freq1, freq2)

  s2.play_tone(duration - 1 #> 1, freq1, freq2)
  s2.play_tone(1, 0, 0)

'---[Read Obstacle Sensors]----------------------------------------------------

PRI ReadObstacle | l, r

  l := s2.obstacle(s2#LEFT, ObstacleThld) & 1
  r := s2.obstacle(s2#RIGHT, ObstacleThld) & 1
  if (l == LeftObstacle and r == RightObstacle)
    ObstacleCount := (Obstaclecount + 1) <# 8
  else
    ObstacleCount := 1
    LeftObstacle := l
    RightObstacle := r

'---[Read Stall Sensor]--------------------------------------------------------

PRI ReadStall

  if (Stalled := s2.stalled & 1)
    StallCount := (StallCount + 1) <# 8
  else
    StallCount := 0

'---[End of Program]-----------------------------------------------------------